'#################################################
'# 
'# Multiple examples for fbTimer
'# 
'# Note that each example is wrapped in a scope, this is not a requirement
'# and is only done here to better separate the examples.
'# 
'#################################################
'# 
'# Add support for events and scheduler via fbList
'#  fbList is available at www.JatteGames.com
#include once "fbList.bi"
'# fbList is only needed if you want to make use of the event -scheduler,
'#  -timeout, and -callback functionality
'# fbList requirement can be disabled by adding: #DEFINE FBTIMER_NOLIST
'# before including fbTimer.bi. Note that this disables fbTimer events
'#  and scheduling! Example:
'#    #define FBTIMER_NOLIST
'#    #include once "fbTimer.bi"
'# 
'#################################################
#include once "fbTimer.bi"


'#################################################
'// This is for these examples only, and is not required code
dim as integer numExamples	=	3
const numEventExamples		=	3
#ifndef FBTIMER_NOLIST '// fbList is included so we have three more examples
	numExamples += numEventExamples
#endif
dim as integer nExample
sleep 50,1 '// Sleep for a bit so runtime values are more interesting
'#################################################



scope
	nExample+=1
	print "Example "& nExample &" / "& numExamples
	print "Creating an instance of timer is as easy as 'dim foo as t_timer'"
	dim as t_timer foo
	print "	Time since foo was created"
	print "		foo       "& foo &" s"
	print "		foo.ms    "& foo.ms &" ms"
	print "		foo.dblms "& foo.dblms &" ms"
	print "		foo.us    "& foo.us &" us"
	print 
	print "	Program runtime"
	print "		foo.Runtime       "& foo.Runtime &" s"
	print "		foo.Runtime.ms    "& foo.Runtime.ms &" ms"
	print "		foo.Runtime.dblms "& foo.Runtime.dblms &" ms"
	print "		foo.Runtime.us    "& foo.Runtime.us &" us"
	print
	print "	Accuracy is "& foo.Accuracy()
	print
	print "Press any key to continue..."
	do
		sleep 10,1
	loop while inkey=""
	cls
end scope





scope
	nExample+=1
	print "Example "& nExample &" / "& numExamples
	dim as t_timer foo
	sleep 1,1 '// Just a short vanilla FB sleep to ensure visible values in the example, NOT REQUIRED FOR CODE TO WORK!
	print "Precision sleep (Does not release CPU time)"
	dim as ulongint before, after
	before = foo.us
	foo.sleep(0.002) '// Sleep for 0.002 ms (Which is 2us)
	after = foo.us
	print "	foo.us before "& before &" us"
	print "	foo.us after  "& after &" us"
	print "		Actual difference: "& after - before &" us"
	
	print 
	print "Press any key to continue..."
	do
		sleep 10,1
	loop while inkey=""
	cls
end scope





scope
	nExample+=1
	print "Example "& nExample &" / "& numExamples
	print "Simple fixed timestep usage example with pause (Press P to toggle)"
	print
	print
	print
	print
	print
	print "Press a key other than P to continue..."
	
	dim as t_timer foo
	dim as integer count
	dim as integer fast
	dim as string userKey
	
	foo.Interval( 1000.0 ) '// Initialize the interval for this timer in dblms
	
	do '// Main loop
		
		if foo.Interval then '// Interval will automatically reset itself based on SetInterval
			'// Any logic here will be executed every '.Interval(milliseconds)' milliseconds
			'// Use .Delta to get the actual precise time between the last two Interval calls in dblms
			print "	(Actual delta "& foo.Delta &" ms)      "
			count += 1
		end if
		
		'// We can perform other logic here, as fast as we want
		locate 4,1
		fast += 1
		print "We're doing this as fast as possible: ["& fast & "]   "
		print "But 'Count' only goes up by 1 per second: 'count' is "& count;
		if foo.isRunning then '// Is foo paused or not?
			print "                 "
		else
			print " - foo is paused!"
		end if
		
		userKey = lcase(inkey)
		if userKey = "p" then
			
			if foo.isRunning then
				foo.Pause
			else
				foo.Resume
			end if
			
		end if
	loop while userKey = "" or userKey = "p"
	cls
end scope





#ifdef FBTIMER_NOLIST
	nExample+=1
	print "Example "& nExample &" to "& (numExamples+numEventExamples) &" cover events and scheduling."
	print " fbTimer events and scheduling requires the optional fbList,"
	print "  AND that FBTIMER_NOLIST is NOT #defined."
	print
	color 14
	print " FBTIMER_NOLIST is currently #defined which has disabled events and scheduling."
	color 15
#else
	'// Scheduling callbacks!
	sub myScheduleCallback(byref event as TimerEvent)
		if event.Count < 5 then
			print "This scheduled event has happened "& event.Count &" times, "& event.Lifetime.ms &" ms since last!"
		else
			print "... has happened "& event.Count &" times, "& event.Lifetime.ms &" ms since last! (Press any key to continue)"
		end if
	end sub
	
	scope
		nExample+=1
		print "Example "& nExample &" / "& numExamples &" [Requires fbList]"
		print "Threadsafe Scheduling example with pause (Press P to toggle)"
		print
		
		dim as t_timer foo
		dim as string userKey
		
		foo.Schedule(1000.0, @myScheduleCallback, TIMER_FOREVER) '// .Schedule(interval, callbackptr, repetitions)
		
		do
			foo.Callbacks()	'// Important! Without this no callbacks will ever occur!
							'// .Callbacks() is also used to control the thread context the callbacks occur in!
			
			userKey = lcase(inkey)
			if userKey = "p" then
				if foo.isRunning then
					foo.Pause
				else
					foo.Resume
				end if
			end if
		loop while userKey = "" or userKey = "p"
		cls
	end scope '// Any remaining Timeout or Scheduled callbacks are cleared when their parent t_Timer goes out of scope
	
	
	
	
	
	'// Timeout callbacks!
	sub myTimeoutCallback(byref event as TimerEvent)
		print "This Timeout happened after "& event.Lifetime.ms &" ms!"
	end sub

	scope
		nExample+=1
		print "Example "& nExample &" / "& numExamples &" [Requires fbList]"
		print "Threadsafe Timeout example:"
		print "  .Timeout() is shorthand for .Schedule() with 1 repetition"
		print
		
		dim as t_timer foo
		
		foo.Timeout(50.0, @myTimeoutCallback)
		foo.Timeout(75.0, @myTimeoutCallback)
		foo.Timeout(150.0, @myTimeoutCallback)
		
		do
			foo.Callbacks()	'// Important! Without this no callbacks will ever occur!
							'// .Callbacks() is also used to control the thread context the callbacks occur in!
			
		loop while foo.Events() > 0 '// There are still events pending
		
		print 
		print "Press any key to continue..."
		do
			sleep 10,1
		loop while inkey=""
		cls
	end scope '// Any remaining Timeout or Scheduled callbacks are cleared when their parent t_Timer goes out of scope
	
	
	
	
	
	'// Timeout and Schedule can be passed user data!
	sub myTimeoutWithUDCallback(byref event as TimerEvent)
		print "This Timeout happened after "& event.Lifetime.ms &" ms! (UserData is "& event.UserData &")"
	end sub
	sub myTimeoutCallbackSingle(byref event as TimerEvent)
		print "This Timeout happened after "& event.Lifetime.ms &" ms! (UserData *ptr is "& *cptr(single ptr, event.UserData) &")"
	end sub

	scope
		nExample+=1
		print "Example "& nExample &" / "& numExamples &" [Requires fbList]"
		print "event.UserData example:"
		print "  Both Timeout and Schedule can be passed user data as the last parameter"
		print
		
		dim as t_timer foo
		dim as single bar = 3.333
		
		foo.Timeout(50.0, @myTimeoutWithUDCallback, 1)		'// UserData can be an integer
		foo.Timeout(75.0, @myTimeoutWithUDCallback, 2)
		foo.Timeout(150.0, @myTimeoutCallbackSingle, @bar)	'// UserData can also be any ptr
		
		do
			foo.Callbacks()	'// Important! Without this no callbacks will ever occur!
							'// .Callbacks() is also used to control the thread context the callbacks occur in!
			
		loop while foo.Events() > 0 '// There are still events pending
		
		print 
		print "Press any key to continue..."
		do
			sleep 10,1
		loop while inkey=""
		cls
	end scope '// Any remaining Timeout or Scheduled callbacks are cleared when their parent t_Timer goes out of scope
#endif

print
print "You've reached the end of the examples!"
sleep
end
